<?php
namespace Models;

// Модель Пользователи
class Users extends \DB\SQL\Mapper {
	public $db, $f3;
	public $accepts='["receipts\/return","receipts\/sales","receipts\/new","receipts\/all_open","receipts\/switch","receipts\/intercept","receipts\/add_line","receipts\/freesales","receipts\/chpok","receipts\/change_line","receipts\/change_price","receipts\/delete_line","receipts\/select_discount","receipts\/discount","partners\/search","receipts\/remove_partner","receipts\/pay","receipts\/fr_no","receipts\/hold","receipts\/search","receipts\/show","receipts\/new_card","receipts\/online_order","products\/form","products\/fast_products","products\/search_barcodes","products\/img","products\/currentstock","receipts\/cashin","receipts\/cashout","receipts\/reports","changes\/open","changes\/search","changes\/x","changes\/z","changes\/close_alert","settings\/reset","settings\/base","settings\/api","settings\/kkm","settings\/customer_display","settings\/scales","settings\/synchronization","settings\/logs","settings\/post_logs","settings\/read_all_logs","settings\/rooms","settings\/tables","api\/products_all","api\/users","api\/partners_all","api\/tickets","api\/barcodes_pr","api\/sno","api\/loyalty","api\/get_remote_receipts","settings\/service","changes\/open_fr","changes\/open_db","changes\/close_fr","changes\/close_db","receipts\/print_last","settings\/status_kkm","settings\/continue_print","settings\/reset_ticket"]';

	public function __construct($db, $f3) {
		parent::__construct($db, 'users' ); // подключаемся к таблицt пользователей
		$this->db=$db;
		$this->f3=$f3;
	}

	// данные о страницах
	function page_data($page) {
		$pages=array(
			'autorization'=>array(
				'title'=>'Авторизация пользователя',
				'inc'=>'users_autorization.htm'
			),
			'autorization_continue'=>array(
				'title'=>'Авторизация пользователя',
				'inc'=>'users_autorization_continue.htm'
			)
		);
		$pages[$page]['page']=$page;

		return $pages[$page];
	}

	// Выбор из базы
	function select_users($query=null, $options=null) {
		if(!$query) {
			$query=array('u.id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$sql="SELECT u.*, g.title AS `group_title`, g.module FROM `users` AS `u` JOIN `usergroups` AS `g` ON g.id=u.group_id WHERE date_delete=0 AND ".$query[0].$option;
		$res=$this->db->exec($sql, $query[1]);
		return $res;
	}

	// Устанавливаем права пользователя
	function users_modules($m) {
		$modules=json_decode($m, true);
		if(!in_array('receipts/reports', $modules)) {
			if(in_array('changes/x', $modules) || in_array('changes/z', $modules) || in_array('changes/search', $modules) || in_array('changes/open', $modules)) {
				$modules[]='receipts/reports';
			}
		}

		// Если можно добавлять товары в чек, то можно и добавлять модификаторы
		if(in_array('receipts/add_line', $modules)) {
			$modules[]='receipts/modifiers';
		}

		// Если можно продавать или возвращать, то можно перемещать чек на другой стол и фискализировать документы удаленно,
		// и указывать маркировку товара
		// и вскрывать бутылки
		if(in_array('receipts/sales', $modules) || in_array('receipts/return', $modules)) {
			$modules[]='receipts/move';
			$modules[]='receipts/print';
			$modules[]='receipts/add_from_agorta';
			$modules[]='receipts/online_order';
			$modules[]='receipts/search_online_order';
			$modules[]='receipts/fiscalization';
			$modules[]='receipts/add_marking';
			$modules[]='receipts/bonus_pay';
		}

		// Если можно изменять кол-во товара, то можно и взвешивать
		if(in_array('receipts/change_line', $modules)) {
			$modules[]='scales/weight';
		}

		// Если можно искать интернет-заказы, то можно их и добавлять
		if(in_array('receipts/online_order', $modules)) {
			$modules[]='receipts/online_order';
			$modules[]='receipts/search_online_order';
		}

		// Если можно настраивать ККМ, то можно настраивать драйвера, принтер повара и пречеков
		if(in_array('settings/kkm', $modules)) {
			$modules[]='settings/drivers';
			$modules[]='settings/start_driver';
			$modules[]='settings/kitchen_printer';
			$modules[]='settings/pre_printer';
			$modules[]='settings/bar_printer';
		}

		// Если разрешен поиск товаров по штрих-коду, то можно и выбирать товары из списка
		if(in_array('products/search_barcodes', $modules)) {
			$modules[]='products/list';
		}

		// Если можно управлять залами, то добавляем соответствующие права на страницы
		if(in_array('settings/rooms', $modules)) {
			$modules[]='rooms/add';
			$modules[]='rooms/delete';
		}

		// Если можно управлять столами, то добавляем соответствующие права на страницы
		if(in_array('settings/tables', $modules)) {
			$modules[]='tables/add';
			$modules[]='tables/delete';
		}

		if(in_array('partners/search', $modules)) {
			$modules[]='partners/search_result';
			$modules[]='receipts/add_partner';
		}

		if(in_array('api/tickets', $modules)) {
			$modules[]='settings/update_ticket_time';
		}
		return $modules;
	}

	// проверка на авторизацию
	function is_login() {
		$res=array(); // массив с информацией об аторизованном пользователе
		$hash=$this->f3->get('COOKIE.hash'); // берем данные из сессии
		if(!$hash) {
			return false;
		}
		$res=$this->select_users(array("u.hash=?", array($hash)), array('limit'=>1));
		if(empty($res)) {
			return false;
		}

		if($res[0]['module']=='') {
			$res[0]['module']=$this->accepts;
		}
		
		$modules=$this->users_modules($res[0]['module']);
		$res[0]['module']=$modules;

		$this->f3->set('authorization_user', $res[0]); // перезапишем информацию об авторизации
		return true;
	}

	// авторизация пользователя
	function user_autorization($post) {
		if(!isset($post['user_pass'])) { // проверяем наличие всех нужных данных
			return array( // если чего-то не хватает, то сообщаем об ошибке
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$res=array();
		// Проверяем что введено логин и пароль или штрих-код сотрудника
		if(mb_strlen($post['user_pass'])==13) {
			$s=new \Models\Settings($this->f3, $this->db);
			$sets=$s->get_settings(array('worker_code_pr'));
			if(mb_substr($post['user_pass'], 0, mb_strlen($sets['worker_code_pr']))==$sets['worker_code_pr']) { // Штрих-код
				// Ищем по штрих коду
				$res=$this->select_users(array('u.card=?', $post['user_pass']), array('limit'=>1));
			}
		}
		elseif(!isset($post['user_id'])) { // проверяем наличие всех нужных данных
			return array( // если чего-то не хватает, то сообщаем об ошибке
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		if(empty($res)) {

			$res=$this->select_users(array('u.id=?', $post['user_id']), array('limit'=>1)); // пытаемся начти выбранного пользователя
			if(empty($res)) { // если его нет, то выводим ошибку
				return array(
					'success'=>false,
					'txt'=>'user_not_found'
				);
			}

			if($res[0]['pass']!='' && $post['user_pass']!=$res[0]['pass']) { // проверяем пароли
				return array(
					'success'=>false,
					'txt'=>'pass_incorrect'
				);
			}
		}

		// Проверяем хэш
		if($post['forcibly']==0 && $res[0]['hash']!='') {
			return array(
				'success'=>false,
				'txt'=>'autorization_is_already_done',
				'next_step'=>'/users/autorization_continue'
			);
		}

		if($res[0]['module']=='') {
			$res[0]['module']=$this->accepts;
		}
		$modules=$this->users_modules($res[0]['module']);
		$res[0]['module']=$modules;

		// Создаем хэш авторизации
		$res[0]['hash']=time().'-'.md5($res[0]['name'].time());
		// Обновляем хэш в базе
		$this->save_user($res[0], 'change');

		$this->f3->set('authorization_user', $res[0]); // если все ок, то запоминаем информацию о пользователе
		$this->f3->set('COOKIE.hash', $res[0]['hash']);
		$url='/';
		if($this->f3->get('QUERY')) {
			$url='/receipts/add_from_agorta?'.$this->f3->get('QUERY');
		}
		return array(
			'success'=>true,
			'txt'=>'login_successful',
			'reroute'=>$url
		);
	}

	// Выход из системы
	function logout() {
		$this->reset();
		$this->load(array('id=?', $this->f3->get('authorization_user.id')), array('limit'=>1));
		if(!$this->dry()) {
			$this->hash='';
			$this->update();
		}
		$this->f3->clear('authorization_user');
		$this->f3->clear('COOKIE.hash');
		$this->f3->clear('COOKIE.receipt_id');
		$this->f3->clear('COOKIE.table_id');
		$this->f3->clear('COOKIE.room_id');
		$this->f3->clear('SESSION');
		return array(
			'success'=>true,
			'txt'=>'logout_successful',
			'reroute'=>'/users/autorization'
		);
	}

	// Сохранение пользователя
	function save_user($data, $action='add') {
		if(!isset($data['name']) || !isset($data['group_id'])) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$this->reset();

		if($action=='change' && $data['id']) {
			$this->load(array('id=?', $data['id']));
		}

		$this->copyFrom($data);
		if($this->save()===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		$this->reset();
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}
}
